# type: ignore
# @auto-fold regex /^\s*if/ /^\s*else/ /^\s*def/
#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# version 0.3.0
# date 29 nov 2022
# sourcery skip

# my coding convention
# **EVAL : evaluate the performance of this method
# **RED  : redo this
# **DEB  : debugging needed in this part
# **DEL  : DELETE AT SOME POINT

import numpy as np
import pandas as pd

import matplotlib.pyplot as pl
import matplotlib.gridspec as gridspec
from importlib import reload

from copy import deepcopy
import os
import logging

from tqdm import tqdm

from reddcolors import Palette

from .model_repo import *
from .unmodel_repo import *
from .block import ReddModel
from .utils import fold_dataframe, nullify_output, flatten


rc = Palette()


def plot_GM_Estimator(estimator, saveloc='', fmt='png', sig_factor=4, fill_cor=0, plot_name='', plot_title=None, plot_ylabel=None):
    # sourcery skip: use-fstring-for-formatting

    cor = ['C0', 'C1', 'C2', 'C4', 'C5', 'C6', 'C7', 'C8', 'C9']
    colors = np.array([cor,cor,cor,cor,cor]).flatten()

    if plot_title is None:
        plot_title = 'Optimal estimate with Gaussian Mixtures\n for '

    if plot_ylabel is None:
        plot_ylabel = 'Probability Density'
    ## COL
    sig_factor = sig_factor
    saveplace = saveloc + '/posteriors/GMEstimates/'
    plot_fmt = fmt
    plot_nm = plot_name


    n_components = estimator.n_components

    mu = estimator.mixture_mean
    var = estimator.mixture_variance
    sig = estimator.mixture_sigma

    xx, yy = [], []

    xticks = [mu]

    fig, ax = pl.subplots(figsize=(8, 4))
    for i in range(sig_factor):
        xx.append(np.array([np.linspace(mu-sig*(i+1),
                                        mu+sig*(i+1),
                                        1000)]).T)

        yy.append(np.exp(estimator.score_samples(xx[i])))

        xx[i] = np.append(np.append(xx[i][0], xx[i]), xx[i][-1])
        yy[i] = np.append(np.append(0, yy[i]), 0)
        ax.fill(xx[i], yy[i], c=colors[fill_cor], alpha=1/sig_factor, zorder=2*(i+1)-1)

        ax.vlines([xx[i][0], xx[i][-1]], ymin=[min(yy[i]), min(yy[i])],
                                         ymax=[yy[i][1], yy[i][-2]],
                                         colors=[rc.fg, rc.fg],
                                         lw=[1.5, 1.5],
                                         ls=['--'],
                                         zorder=2*(i+1))

        xticks.extend((xx[i][0], xx[i][-1]))
    ax.vlines(mu, ymin=[min(yy[0])],
                  ymax=[np.exp(estimator.score_samples([[mu]]))],
                  colors=[rc.fg],
                  lw=[2],
                  ls=['-'],
                  zorder=2*sig_factor)


    ax.plot(xx[-1], yy[-1], c=rc.fg, alpha=1, lw=2, zorder=9)

    mu_display = np.round(mu, 3)
    sig_display = np.round(sig, 3)

    # Dummy plots for labels
    ax.plot(
        np.median(xx[0]),
        np.median(yy[0]),
        alpha=0.0,
        label=f'$\mu = {mu_display}$',
    )
    ax.plot(np.median(xx[0]), np.median(yy[0]), alpha=0., label=r'$\sigma = {}$'.format(sig_display))
    if n_components > 1:
        ax.plot(np.median(xx[0]), np.median(yy[-1]), alpha=0., label=r'$N = {}$'.format(n_components))

    # Set ticks, labels, title, etc
    xticks = np.sort(xticks)

    dif = xticks[-1] - xticks[0]
    nround = 2
    for i in range(4):
        nround = i + 2
        if np.round(dif, i) // 10**-i > 0:
            break

    xticks = np.round(xticks, nround)
    yticks = np.round(np.linspace(0, max(yy[0]), 5), 2)

    ax.tick_params(axis='x', labelrotation=45)
    ax.set_xticks(xticks, minor=False)
    ax.set_yticks(yticks, minor=False)
    ax.legend(framealpha=0.)
    ax.set_title(plot_title+'{}'.format(plot_nm))
    ax.set_xlabel('{} [{}]'.format(plot_nm, estimator.unit))
    ax.set_ylabel(plot_ylabel)


    pl.savefig(saveplace+'{}.{}'.format(plot_nm, plot_fmt),
               bbox_inches='tight')

    pl.close('all')


def plot_traces(sampler, eng_name, my_model, saveloc='', trace_modes=None, fmt='png'):
    if trace_modes is None:
        trace_modes = [0]
    # 0:trace, 1:norm_post, 2:dens_interv, 3:corner
    trace_mode_dic = {0:'Trace Plot',
                      1:'Normalised Posterior',
                      2:'Density Interval',
                      3:'Corner Plot'}
    saveplace = saveloc + '/traces/'

    cor = ['C0', 'C1', 'C2', 'C4', 'C5', 'C6', 'C7', 'C8', 'C9']
    colors_ = np.array([cor,cor,cor,cor,cor]).flatten()
    vn = np.array(flatten(my_model.get_attr_param('name')))[my_model.C_]
    for trace_mode in trace_modes:
        try:
            if eng_name == 'reddemcee':
                import arviz as az
                arviz_data = az.from_emcee(sampler=sampler,
                                            var_names=vn)

                # trace
                if trace_mode == 0:
                    circ_mask = np.array(flatten(my_model.get_attr_param('is_circular')))[my_model.C_]
                    circ_var_names = vn[circ_mask]
                    for b in my_model:
                        vn_b = np.array(b.get_attr('name'))[b.C_]
                        az.plot_trace(arviz_data,
                                    figsize=(14, len(vn_b)*2.5),
                                    var_names=vn_b,
                                    circ_var_names=circ_var_names,
                                    plot_kwargs={'color':rc.fg},
                                    trace_kwargs={'color':rc.fg})

                        pl.subplots_adjust(hspace=0.60)
                        savefigname = saveplace+'{} {}.{}'.format(trace_mode_dic[trace_mode], b.name_, fmt)
                        pl.savefig(savefigname)

                # distr
                elif trace_mode == 1:
                    for b in my_model:
                        for p in b[b.C_]:
                            fig, ax = pl.subplots(1, 1)
                            fig.suptitle(p.name)

                            az.plot_dist(arviz_data.posterior[p.name].values,
                                        color=rc.fg,
                                        rug=True,
                                        #figsize=(8, 6),
                                        )
                            #pl.ylabel('Probability Density')
                            pl.xlabel('Value')

                            savefigname = saveplace+'{} {}.{}'.format(trace_mode_dic[trace_mode], p.name, fmt)
                            pl.savefig(savefigname)
                # density intervals
                elif trace_mode == 2:
                    for b in my_model:
                        axes = az.plot_density(
                            [arviz_data],
                            var_names=np.array(b.get_attr('name'))[b.C_],
                            shade=0.2,
                            colors=colors_[b.bnumber_-1],
                            #hdi_markers='v'
                            )

                        fig = axes.flatten()[0].get_figure()
                        fig.suptitle("94% High Density Intervals")

                        savefigname = saveplace+'{} {}.{}'.format(trace_mode_dic[trace_mode], b.name_, fmt)
                        pl.savefig(savefigname)
                # corner plot
                elif trace_mode == 3:
                    ax = az.plot_pair(arviz_data,
                            kind=["scatter", "kde"],
                            marginals=True,
                            marginal_kwargs={'color':rc.fg},
                            point_estimate="median",
                            scatter_kwargs={'color':rc.fg},
                            point_estimate_kwargs={'color':'red'},
                            point_estimate_marker_kwargs={'color':'red',
                                                        's':200,
                                                        'alpha':0.75},
                            )

                    savefigname = saveplace+'{}.{}'.format(trace_mode_dic[trace_mode], fmt)
                    pl.savefig(savefigname)

            elif eng_name == 'dynesty':
                from dynesty import plotting as dyplot
                res2 = sampler

                if trace_mode == 0:
                    # trace
                    for b in my_model:
                        vnb = np.array(b.get_attr('name'))[b.C_]
                        fig, axes = dyplot.traceplot(res2,
                                                    post_color=rc.fg,
                                                    trace_color=rc.fg,
                                                    labels=vnb,
                                                    dims=b.slice_true)
                        savefigname = saveplace+'{} {}.{}'.format(trace_mode_dic[trace_mode], b.name_, fmt)
                        pl.savefig(savefigname)
                # Normalised Posterior
                elif trace_mode == 1:   
                    arviz_data = az.from_emcee(sampler=sampler,
                                                var_names=vn)

                    for b in my_model:
                        for p in b[b.C_]:
                            fig, ax = pl.subplots(1, 1)
                            fig.suptitle(p.name)

                            az.plot_dist(arviz_data.posterior[p.name].values)

                            savefigname = saveplace+'{} {}.{}'.format(trace_mode_dic[trace_mode], p.name, fmt)
                            pl.savefig(savefigname)
                # density intervals
                elif trace_mode == 2:
                    arviz_data = az.from_emcee(sampler=sampler,
                                                var_names=vn)

                    for b in my_model:
                        axes = az.plot_density(
                            [arviz_data],
                            var_names=np.array(b.get_attr('name'))[b.C_],
                            shade=0.2,
                            #hdi_markers='v'
                            )

                        fig = axes.flatten()[0].get_figure()
                        fig.suptitle("94% High Density Intervals")

                        savefigname = saveplace+'{} {}.{}'.format(trace_mode_dic[trace_mode], b.name_, fmt)
                        pl.savefig(savefigname)
                # corner plot
                elif trace_mode == 3:
                    arviz_data = az.from_emcee(sampler=sampler,
                                                var_names=vn)

                    ax = az.plot_pair(arviz_data,
                            kind=["scatter", "kde"],
                            marginals=True,
                            marginal_kwargs={'color':rc.fg},
                            point_estimate="median",
                            scatter_kwargs={'color':rc.fg},
                            point_estimate_kwargs={'color':'red'},
                            point_estimate_marker_kwargs={'color':'red',
                                                        's':90},
                            )
                    savefigname = saveplace+'{}.{}'.format(trace_mode_dic[trace_mode], fmt)
                    pl.savefig(savefigname)

            else:
                print('Method is not yet implemented for {}'.format(eng_name))
                return None

            pl.close('all')
        except:
            print(f'Trace plot for {trace_mode_dic[trace_mode]} failed!')


def plot_KeplerianModel(my_data, my_model, res, saveloc='', options=None):
    if options is None:
        options = {}
    if True:
        saveplace = saveloc + '/models/'
        unsaveplace = saveloc + '/models/uncertainpy/'

        plot_fmt = options['format']
        switch_histogram = options['hist']
        switch_uncertain = options['uncertain']
        switch_errors = options['errors']
        logger_level = options['logger_level']
        gC = options['gC']


        posterior_method = 'GM'
        c = ['C0', 'C1', 'C2', 'C4', 'C5', 'C6', 'C7', 'C8', 'C9']
        colors = np.array([c,c,c,c,c]).flatten()

        temp_file_names = []
        temp_mod_names = []
        temp_dat_names = []

        if switch_uncertain:
            import uncertainpy as un
            import chaospy as cp
            if logger_level is not None:
                logging.getLogger("chaospy").setLevel(logger_level)
                logging.getLogger("numpoly").setLevel(logger_level)


    def create_mod(data_arg, blocks_arg, tail_x, mod_number=0):
        x = ReddModel(data_arg, blocks_arg)
        x.A_ = []

        temp_script = 'temp_mod_0{}.py'.format(mod_number)
        temp_file_names.append(temp_script)
        temp_mod_names.append('{}/temp/temp_model_{}{}.py'.format(saveloc, x.model_script_no, tail_x))
        temp_dat_names.append('{}/temp/temp_data{}.csv'.format(saveloc, tail_x))

        with open(temp_script, 'w') as f:
            f.write(open('support/init.scr').read())
            # DEPENDENCIES
            f.write('''
import kepler
''')
            # CONSTANTS
            f.write('''
nan = np.nan
A_ = []
mod_fixed_ = []
gaussian_mixture_objects = dict()
''')

            f.write(open(x.write_model_(loc=saveloc, tail=tail_x)).read())

    if True:
        ## COL
        D = deepcopy(my_data)
        ajuste = res
        OGM = my_model

        # Block selection
        DB_A = [b for b in OGM if b.display_on_data_==True]
        NDB_A = [b for b in OGM if b.display_on_data_==False]

        pbar_tot = 1 + len(DB_A)

        pbar = tqdm(total=pbar_tot)
        # data for continuous
        x_c = np.linspace(D['BJD'].min(), D['BJD'].max(), 5000)
        DC = pd.DataFrame({'BJD':x_c, 'RV':np.zeros_like(x_c), 'eRV':np.zeros_like(x_c)})
        ### MOVE DATA AROUND
        # Get models which I dont want to display with a line
        # Substract them to the data
        create_mod(D, NDB_A, '_NDM_A', 0)


        import temp_mod_00
        NDM_A_mod = reload(temp_mod_00).my_model


        ndm, ndferr = NDM_A_mod(ajuste)

        # data gets cleaned from no display
        D['RV'] -= ndm.values
        D['eRV'] = np.sqrt(D['eRV'].values**2 + ndferr.values)

        # get the residuals

        create_mod(D, DB_A, '_DM_A', 1)

        import temp_mod_01
        DM_A_mod = reload(temp_mod_01).my_model

        D['residuals'] = D['RV'].values - DM_A_mod(ajuste)[0]

        # Here we are done. We have 3 different datasets
        # display, og-no_display, continuous

    # FULL MODEL
    # this requires a subgrid, for plotting the residuals

    if True:
        if True:
            fig = pl.figure(figsize=(10, 8))
            gs = gridspec.GridSpec(3, 4)
            if switch_histogram:
                ax = fig.add_subplot(gs[:2, :-1])
                axr = fig.add_subplot(gs[2, :-1], sharex=ax)
                axh = fig.add_subplot(gs[:2, 3], sharey=ax)
                axrh = fig.add_subplot(gs[2, 3], sharey=axr)

            else:
                ax = fig.add_subplot(gs[:2, :])
                axr = fig.add_subplot(gs[2, :], sharex=ax)

            pl.subplots_adjust(hspace=0)

            ax.axhline(0, color='gray', linewidth=2)
            axr.axhline(0, color='gray', linewidth=2)

            pl.subplots_adjust(wspace=0.15)

        # First we plot the data
        if True:
            for b_ins in NDB_A:
                if b_ins.type_ == 'Instrumental':
                    mask = D['Flag'] == b_ins.number_
                    if switch_errors:
                        ax.errorbar(D[mask]['BJD'], D[mask]['RV'], D[mask]['eRV'],
                                c=colors[b_ins.number_-1], marker='o', label=b_ins.instrument_label,
                                ls='')

                        axr.errorbar(D[mask]['BJD'], D[mask]['residuals'], D[mask]['eRV'],
                                c=colors[b_ins.number_-1], marker='o',
                                ls='')
                    else:
                        ax.plot(D[mask]['BJD'], D[mask]['RV'],
                                colors[b_ins.number_-1]+'o', label=b_ins.instrument_label)

                        axr.plot(D[mask]['BJD'], D[mask]['residuals'], colors[b_ins.number_-1]+'o')
        # We set unmodels for uncertainties
        if True:
            for b in DB_A:
                if b.parameterisation == 0:
                    kepmod = Keplerian_Model
                    unkepmod = unKeplerian_Model
                    un_model_name = "unKeplerian_Model"
                    chaos_names = ['Period', 'Amplitude', 'Phase', 'Eccentricity', 'Longitude_Periastron']
                if b.parameterisation == 1:
                    kepmod = Keplerian_Model_1
                    unkepmod = unKeplerian_Model_1
                    un_model_name = "unKeplerian_Model_1"
                    chaos_names = ['lPeriod', 'Amp_sin', 'Amp_cos', 'Ecc_sin', 'Ecc_cos']
                if b.parameterisation == 2:
                    kepmod = Keplerian_Model_2
                    unkepmod = unKeplerian_Model_2
                    un_model_name = "unKeplerian_Model_2"
                    chaos_names = ['Period', 'Amplitude', 'Time_Periastron', 'Eccentricity', 'Longitude_Periastron']
                if b.parameterisation == 3:
                    kepmod = Keplerian_Model_3
                    unkepmod = unKeplerian_Model_3
                    un_model_name = "unKeplerian_Model_3"
                    chaos_names = ['Period', 'Amplitude', 'Time_Periastron', 'Ecc_sin', 'Ecc_cos']

        # Now we plot our line
        DB_AC = deepcopy(DB_A)

        create_mod(DC, DB_AC, '_DM_AC', 2)

        import temp_mod_02
        DM_AC_mod = reload(temp_mod_02).my_model


        DC['RV'] = DM_AC_mod(ajuste)[0]

        ax.plot(DC['BJD'], DC['RV'], color=rc.fg, ls='--')

        if True and switch_histogram:
            nbins = 5
            while nbins < len(D):
                counts, bins = np.histogram(D['RV'], bins=nbins)
                if (counts==0).any():
                    break
                else:
                    nbins += 1

            nbins = 5
            while nbins < len(D):
                counts, bins = np.histogram(D['residuals'], bins=nbins)
                if (counts==0).any():
                    break
                else:
                    nbins += 1

            # PLOT HISTOGRAMS
            axh.hist(D['RV'], bins=nbins-1, orientation='horizontal', ec=rc.fg, lw=1)
            axrh.hist(D['residuals'], bins=nbins-1, orientation='horizontal', ec=rc.fg, lw=1)
            # HIDE TICKS
            axh.tick_params(axis="x", labelbottom=False)
            axh.tick_params(axis="y", labelleft=False)
            axrh.tick_params(axis="y", labelleft=False)

            axrh.set_xlabel('Counts')
        # Ticks and labels
        if True:
            ax.tick_params(axis="x", labelbottom=False)

            ax.set_title('Keplerian Model')
            ax.set_ylabel(r'RVs $\frac{m}{s}$')
            ax.legend(fontsize=10)#, framealpha=0)

            axr.set_xlabel('BJD (days)')
            axr.set_ylabel(r'Residuals $\frac{m}{s}$')

            pl.savefig(saveplace+'{}.{}'.format('keplerian_model', plot_fmt),
                       bbox_inches='tight')

            # here we plot the phasefolded versions

        # we also need a grid for the residuals
        pbar.update(1)


    # PHASEFOLD
    for mode in range(True+switch_uncertain):
        # add a tail to the name if uncertainties
        if mode == 0:
            name_tail = ''
        if mode == 1:
            name_tail = '_uncertainties'
            pbar_tot = len(DB_A)
            pbar = tqdm(total=pbar_tot)
        chaos_thetas = []
        nb_ = 0
        for b in DB_A:
            # make grid
            if True:
                fig = pl.figure(figsize=(10, 8))
                gs = gridspec.GridSpec(3, 4)

                if switch_histogram:
                    ax = fig.add_subplot(gs[:2, :-1])
                    axr = fig.add_subplot(gs[2, :-1], sharex=ax)
                    axh = fig.add_subplot(gs[:2, 3], sharey=ax)
                    axrh = fig.add_subplot(gs[2, 3], sharey=axr)
                else:
                    ax = fig.add_subplot(gs[:2, :])
                    axr = fig.add_subplot(gs[2, :], sharex=ax)

                pl.subplots_adjust(hspace=0)

                ax.axhline(0, color='gray', linewidth=2)
                axr.axhline(0, color='gray', linewidth=2)

            pl.subplots_adjust(wspace=0.15)

            # adjust params for different parameterisations
            # now just in period for param 1
            # Get PAE? ## RED
            per = np.exp(b[0].value) if b.parameterisation == 1 else b[0].value
            D_PF = deepcopy(D)

            TB = [deepcopy(b)]

            if True:
                create_mod(D_PF, TB, '_TB', 3)
                import temp_mod_03
                TM_mod = reload(temp_mod_03).my_model

            D_PF['RV_TB'] = TM_mod(ajuste)[0]
            D_PF['RV_D'] = D_PF['residuals'].values + D_PF['RV_TB'].values
            D_PF['eRV_D'] = TM_mod(ajuste)[1]

            D_PF = fold_dataframe(D_PF, per=per)

            ## plot data per instrument
            for b_ins in NDB_A:
                if b_ins.type_ == 'Instrumental':
                    mask = D_PF['Flag']==b_ins.number_
                    
                    if switch_errors:
                        ax.errorbar(D_PF[mask]['BJD'], D_PF[mask]['RV_D'], yerr=D_PF[mask]['eRV'],
                                    c=colors[b_ins.number_-1], marker='o', ls='', label=b_ins.instrument_label)

                        axr.errorbar(D_PF[mask]['BJD'], D_PF[mask]['residuals'], yerr=D_PF[mask]['eRV'],
                                c=colors[b_ins.number_-1], marker='o', ls='')
                    else:
                        ax.plot(D_PF[mask]['BJD'], D_PF[mask]['RV_D'],
                                colors[b_ins.number_-1]+'o', label=b_ins.instrument_label)

                        axr.plot(D_PF[mask]['BJD'], D_PF[mask]['residuals'],
                                colors[b_ins.number_-1]+'o')

            ### create model line for the phasefold
            xpf_c = np.linspace(D_PF['BJD'].min(), D_PF['BJD'].max(), 5000)
            D_PFC = pd.DataFrame({'BJD':xpf_c, 'RV':np.zeros_like(xpf_c), 'eRV':np.zeros_like(xpf_c)})

            TB_C = [deepcopy(b)]

            if True:
                create_mod(D_PFC, TB_C, '_TB_C', 4)
                import temp_mod_04
                TM_C_mod = reload(temp_mod_04).my_model

            D_PFC['RV'] = TM_C_mod(ajuste)[0]


            ## plot phasefold
            if True:
                # get uncertainties
                if mode==1:
                    un_model = un.Model(run=unkepmod,
                                        labels=['BJD (days)', r'RVs $\frac{m}{s}$'],
                                        interpolate=True,
                                        logger_level=u'error',
                                        #postprocess=func
                                        )

                    chaostheta = {}
                    for i in range(len(b)):
                        if posterior_method == 'GM':
                            if b[i].fixed is None:
                                chaostheta[chaos_names[i]] = cp.TruncNormal(lower=b[i].limits[0],
                                                                            upper=b[i].limits[1],
                                                                            mu=b[i].posterior.mixture_mean,
                                                                            sigma=b[i].posterior.mixture_sigma)
                            else:
                                chaostheta[chaos_names[i]] = b[i].value_mean

                    chaos_thetas.append(chaostheta)

                    parameters = un.Parameters(chaostheta)

                    UQ = un.UncertaintyQuantification(model=un_model,
                                                      parameters=parameters,
                                                      logger_level=u'critical',
                                                      logger_filename=unsaveplace+'uncertainpy.log')

                    with nullify_output(suppress_stdout=True, suppress_stderr=True):
                        undata = UQ.quantify(seed=10,
                                             method='pc',
                                             plot=None,
                                             #plot='all',
                                             pc_method='collocation',
                                             logger_level=u'critical',
                                             figure_folder=unsaveplace+'figures',
                                             data_folder=unsaveplace+'data',
                                             single=False)

                    keplerian_data = un.Data('{}data/{}.h5'.format(unsaveplace,
                                                                   un_model_name))

                    untime = undata[un_model_name].time
                    unmean = undata[un_model_name].mean
                    unvariance = undata[un_model_name].variance
                    unpercentile_5 = undata[un_model_name].percentile_5
                    unpercentile_95 = undata[un_model_name].percentile_95
                    unsensitivity = undata[un_model_name].sobol_first

                # Plot model line
                ax.plot(D_PFC['BJD'].values, D_PFC['RV'].values, color=rc.fg, ls='--')

                # plot uncertainties
                if mode == 1:
                    ax.fill_between(untime,
                                    unpercentile_5,
                                    unpercentile_95,
                                    color=rc.fg,
                                    alpha=0.5)


                    if True:
                        figs, axs = pl.subplots(figsize=(8, 6))
                        for i in range(unsensitivity.shape[0]):
                            axs.plot(untime, unsensitivity[i],
                                       linewidth=1.5,
                                       )
                        axs.set_title('First-order Sobol indices')
                        axs.set_xlabel('BJD (days)')
                        axs.set_ylabel('First-order Sobol indices')
                        axs.legend(parameters.get_from_uncertain(),
                                            loc='upper right',
                                            framealpha=0.5)

                        figs.savefig(saveplace+'sobol_{}.{}'.format(b.name_+name_tail, plot_fmt),
                                   bbox_inches='tight')
                        pl.close(figs)
                # Plot Histogram Model
                if switch_histogram:
                    nbins = 5
                    while nbins < len(D_PF):
                        counts, bins = np.histogram(D_PF['RV_D'], bins=nbins)
                        if (counts==0).any():
                            break
                        else:
                            nbins += 1

                    axh.hist(D_PF['RV_D'], bins=nbins-1, orientation='horizontal', ec=rc.fg, lw=1)

                # Plot histogram residuals
                if switch_histogram:
                    nbins = 5
                    while nbins < len(D_PF):
                            counts, bins = np.histogram(D_PF['residuals'], bins=nbins)
                            if (counts==0).any():
                                break
                            else:
                                nbins += 1
                    axrh.hist(D_PF['residuals'], bins=nbins-1, orientation='horizontal', ec=rc.fg, lw=1)

                # Ticks and labels

                if True:
                    ax.tick_params(axis="x", labelbottom=False)

                    if switch_histogram:
                        axh.tick_params(axis="x", labelbottom=False)
                        axh.tick_params(axis="y", labelleft=False)
                        axrh.tick_params(axis="y", labelleft=False)

                    ax.set_title('Keplerian Model')
                    ax.set_ylabel(r'RVs $\frac{m}{s}$')
                    ax.legend(fontsize=10)#, framealpha=0)

                    axr.set_xlabel('BJD (days)')
                    axr.set_ylabel(r'Residuals $\frac{m}{s}$')



            pl.savefig(saveplace+'{}.{}'.format(b.name_+name_tail, plot_fmt),
                       bbox_inches='tight')

            nb_ += 1

            pbar.update(1)


            # print('MARKER 5')
            if nb_ == len(DB_A):
                pbar.close()

    temp_file_folder = saveloc+'/models/temp/'


    with nullify_output():
        for file in list(set(temp_file_names)):
            try:
                os.system('mv {0} {1}{0}'.format(file, temp_file_folder))
            except Warning:
                print('Couldnt auto-delete temp files')

        for file in list(set(temp_mod_names)):
            try:
                os.system('mv {0} {1}{2}'.format(file, temp_file_folder, file.split('/')[-1]))
            except Warning:
                print('Couldnt auto-delete temp files')

        for file in list(set(temp_dat_names)):
            try:
                os.system('mv {0} {1}{2}'.format(file, temp_file_folder, file.split('/')[-1]))
            except Warning:
                print('Couldnt auto-delete temp files')

    return chaos_thetas









# blue verde rojo purp naran cyan brown blue green



# @auto-fold regex /^\s*if/ /^\s*else/ /^\s*def/
#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# version 0.3
# date 14 nov 2022

# my coding convention
# **EVAL : evaluate the performance of this method
# **RED  : redo this
# **DEB  : debugging needed in this part
# **DEL  : DELETE AT SOME POINT
# **FIN  : Finish this

import numpy as np
from .block import Parameter, Parameter_Block
from .utils import ModelWrapper
from .model_repo import *


def mk_KeplerianBlock(my_data, parameterisation=0, number=1):
    my_params = []
    subscript_nums = ['', '₁', '₂', '₃', '₄', '₅', '₆', '₇', '₈', '₉']

    if parameterisation == 0:
        kepmod = Keplerian_Model
        pnames = ['Period', 'Amplitude', 'Phase', 'Eccentricity', 'Longitude']
        punits = ['(Days)', r'($\frac{m}{s}$)', '(rad)', '', '(rad)']
        math_display = f'K⋅(cos(ν(t,P,𝜙,e)+𝜔)+e⋅cos(𝜔))|{subscript_nums[number]}'
        b_script = 'kep00.model'
        pis_circular = [False, False, True, False, True]
        pis_hou = [False, False, False, False, False]

    elif parameterisation == 1:
        kepmod = Keplerian_Model_1
        pnames = ['lPeriod', 'Amp_sin', 'Amp_cos', 'Ecc_sin', 'Ecc_cos']
        punits = ['(Days)', r'($\frac{m}{s}$)', '(rad)', '', '(rad)']
        math_display = f'K⋅(cos(ν(t,P,𝜙,e)+𝜔)+e⋅cos(𝜔))|{subscript_nums[number]}'
        b_script = 'kep01.model'
        pis_circular = [False, False, False, False, False]
        pis_hou = [False, True, True, True, True]

    elif parameterisation == 2:
        kepmod = Keplerian_Model_2
        pnames = ['Period', 'Amplitude', 'T_0', 'Eccentricity', 'Longitude']
        punits = ['(Days)', r'($\frac{m}{s}$)', '(Days)', '', '(rad)']
        math_display = f'K⋅(cos(ν(t,P,T₀,e)+𝜔)+e⋅cos(𝜔))|{subscript_nums[number]}'
        b_script = 'kep02.model'
        pis_circular = [False, False, False, False, True]
        pis_hou = [False, False, False, False, False]

    elif parameterisation == 3:
        kepmod = Keplerian_Model_3
        pnames = ['Period', 'Amplitude', 'T_0', 'Ecc_sin', 'Ecc_cos']
        punits = ['(Days)', r'($\frac{m}{s}$)', '(Days)', '', '(rad)']
        math_display = f'K⋅(cos(ν(t,P,T₀,e)+𝜔)+e⋅cos(𝜔))|{subscript_nums[number]}'
        b_script = 'kep03.model'
        pis_circular = [False, False, False, False, False]
        pis_hou = [False, False, False, True, True]

    if number:
        pnames = [pnam+' %i' % number for pnam in pnames]

    bdim = len(pnames)

    pvalues = [-np.inf for _ in range(bdim)]
    ppriors = ['Uniform' for _ in range(bdim)]
    plimits = [[None, None] for _ in range(bdim)]

    ptypes = [None for _ in range(bdim)]
    prargs = [None for _ in range(bdim)]
    ptformargs = [None for _ in range(bdim)]
    pfixed = [None for _ in range(bdim)]
    psigma = [None for _ in range(bdim)]

    pGM_parameter = [None for _ in range(bdim)]
    pposterior = [None for _ in range(bdim)]

    pstds = [None for _ in range(bdim)]

    for i in range(bdim):
        d0 = {'name':pnames[i], 'prior':ppriors[i], 'value':pvalues[i],
                  'limits':plimits[i], 'unit':punits[i], 'prargs':prargs[i],
                  'type':ptypes[i], 'ptformargs':ptformargs[i], 'fixed':pfixed[i],
                  'sigma':psigma[i], 'GM_parameter':pGM_parameter[i],
                  'posterior':pposterior[i], 'std':pstds[i],
                  'is_circular':pis_circular[i],
                  'is_hou':pis_hou[i]}

        my_params.append(Parameter(d0))

    b_mod = ModelWrapper(kepmod, [my_data.values[:, 0]])
    b_name = f'KeplerianBlock {number}'

    return Parameter_Block(my_params, block_model=b_mod,
                           block_name=b_name, block_type='Keplerian',
                           model_script=b_script,
                           is_iterative=True,
                           math_display=math_display, display=True,
                           parameterisation=parameterisation, number=number)


def mk_InstrumentBlock(my_data, number=1, moav=0, sa=False):
    my_params = []
    #subscript_nums = ['', '₁', '₂', '₃', '₄', '₅', '₆', '₇', '₈', '₉']
    supscript_nums = ['', '¹', '²', '³', '⁴', '⁵', '⁶', '⁷', '⁸', '⁹']

    from model_repo import Instrument_Model as insmod
    pnames = ['Offset', 'Jitter']
    punits = [r'($\frac{m}{s}$)', r'($\frac{m}{s}$)']
    math_display = f'γ₀|{supscript_nums[number]}'
    if number:
        pnames = [pnam+' %i' % number for pnam in pnames]
    if moav > 0:
        for j in range(moav):
            if number:
                pnames.append(f'MACoefficient {number} Order {j + 1}')
                pnames.append(f'MATimescale {number} Order {j+1}')

                punits.append('(Days)')
            else:
                pnames.extend((f'MACoefficient Order {j + 1}', f'MATimescale Order {j + 1}'))
        math_display = f'{math_display} + 𝛴ᵢ𝛴ₘ 𝛷ₘ⋅exp((t₍ᵢ₋ₘ₎-tᵢ)/𝜏ₘ)⋅𝜀(t₍ᵢ₋ₘ₎)'

    bdim = len(pnames)

    pvalues = [-np.inf for _ in range(bdim)]
    ppriors = ['Uniform' for _ in range(bdim)]
    plimits = [[None, None] for _ in range(bdim)]

    ptypes = [None for _ in range(bdim)]
    prargs = [None for _ in range(bdim)]
    ptformargs = [None for _ in range(bdim)]
    pfixed = [None for _ in range(bdim)]
    psigma = [None for _ in range(bdim)]

    pGM_parameter = [None for _ in range(bdim)]
    pposterior = [None for _ in range(bdim)]

    pstds = [None for _ in range(bdim)]
    pis_circular = [False for _ in range(bdim)]
    pis_hou = [False for _ in range(bdim)]

    for i in range(bdim):
        d0 = {'name':pnames[i], 'prior':ppriors[i], 'value':pvalues[i],
                  'limits':plimits[i], 'unit':punits[i], 'prargs':prargs[i],
                  'type':ptypes[i], 'ptformargs':ptformargs[i], 'fixed':pfixed[i],
                  'sigma':psigma[i], 'GM_parameter':pGM_parameter[i],
                  'posterior':pposterior[i], 'std':pstds[i],
                  'is_circular':pis_circular[i],
                  'is_hou':pis_hou[i]}

        my_params.append(Parameter(d0))


    if moav > 0:
        from model_repo import Instrument_Moav_Model as insmod
        b_mod = ModelWrapper(insmod, [number, my_data.values[:, 3], moav,
                             my_data.values[:, 0], my_data.values[:, 1]])
        b_script = 'ins01.model'
        if sa:
            from model_repo import Instrument_Moav_SA_Model as insmod
            b_mod = ModelWrapper(insmod, [number, my_data.values[:, 3], moav,
                                 my_data.values[:, 0], my_data.values[:, 4:],
                                 my_data.shape[1]-4])
            b_script = None
    else:
        from model_repo import Instrument_Model as insmod
        b_mod = ModelWrapper(insmod, [number, my_data.values[:, 3]])
        b_script = 'ins00.model'

    b_name = f'InstrumentalBlock {number}'

    return Parameter_Block(my_params, block_model=b_mod,
                           block_name=b_name, block_type='Instrumental',
                           model_script=b_script,
                           is_iterative=False,
                           math_display=math_display, display=False,
                           parameterisation=None, number=number,
                           moav=moav)


def mk_AccelerationBlock(my_data, n=1):
    my_params = []
    subscript_nums = ['', '₁', '₂', '₃', '₄', '₅', '₆', '₇', '₈', '₉']
    #supscript_nums = ['', '¹', '²', '³', '⁴', '⁵', '⁶', '⁷', '⁸', '⁹']
    punits = []
    from model_repo import Acceleration_Model as acmod
    for i in range(n):
        if i == 0:
            pnames = ['Acceleration']
            punits.append(r'($\frac{m}{s^2}$)')
        else:
            pnames.append(f'Acceleration Order {str(i + 1)}')
            punits.append(r'($\frac{m}{s^%s}$)' % str(i+2))

    bdim = len(pnames)


    pvalues = [-np.inf for _ in range(bdim)]
    ppriors = ['Uniform' for _ in range(bdim)]
    yearly = 1/365.25
    plimits = [[-yearly, yearly] for _ in range(bdim)]

    ptypes = [None for _ in range(bdim)]
    prargs = [None for _ in range(bdim)]
    ptformargs = [None for _ in range(bdim)]
    pfixed = [None for _ in range(bdim)]
    psigma = [None for _ in range(bdim)]

    pGM_parameter = [None for _ in range(bdim)]
    pposterior = [None for _ in range(bdim)]

    pstds = [None for _ in range(bdim)]
    pis_circular = [False for _ in range(bdim)]
    pis_hou = [False for _ in range(bdim)]

    math_display = f'γ{subscript_nums[1]}'
    for j in range(bdim-1):
        math_display += f' + γ{subscript_nums[2 + j]}'
    for i in range(bdim):
        d0 = {'name':pnames[i], 'prior':ppriors[i], 'value':pvalues[i],
                  'limits':plimits[i], 'unit':punits[i], 'prargs':prargs[i],
                  'type':ptypes[i], 'ptformargs':ptformargs[i], 'fixed':pfixed[i],
                  'sigma':psigma[i], 'GM_parameter':pGM_parameter[i],
                  'posterior':pposterior[i], 'std':pstds[i],
                  'is_circular':pis_circular[i],
                  'is_hou':pis_hou[i]}


        my_params.append(Parameter(d0))


    b_mod = ModelWrapper(acmod, [my_data.values[:, 0]])
    b_name = f'AccelerationBlock o{n}'
    b_script = 'acc.model'

    return Parameter_Block(my_params, block_model=b_mod,
                           block_name=b_name, block_type='Acceleration',
                           model_script=b_script,
                           is_iterative=False,
                           math_display=math_display, display=False,
                           parameterisation=None, number=n)


def mk_AdditionalPriorsBlock(my_data):
    my_params = []

    from model_repo import Empty_Model as empmod
    b_mod = ModelWrapper(empmod, [my_data.values[:, 0]])
    b_name = 'AdditionalPriorsBlock'

    math_display = ''
    return Parameter_Block(my_params, block_model=b_mod,
                           block_name=b_name, block_type='AdditionalPriors',
                           is_iterative=False,
                           math_display=math_display, display=False,
                           parameterisation=None, number=None)


def SmartLimits(my_data, b, *args, **kwargs):
    uni = 'Uniform'
    norm = 'Normal'
    # ecc limits and prargs
    additional_priors = []
    lims = []
    priors = []
    prargs = []
    d = kwargs

    if b.type_ == 'Keplerian':
        sig_limiter = my_data['RV'].std(ddof=0)
        per_limiter = my_data['BJD'].max() - my_data['BJD'].min()
        amp_limiter = sig_limiter * np.sqrt(3)
        angle_limits = [0, 2*np.pi]

        ecc_limits, ecc_prargs = d['prargs']

        if b.parameterisation == 0:
            lims = [[0.1, per_limiter], [0, amp_limiter], angle_limits,
                       ecc_limits, angle_limits]
            priors = [uni, uni, uni, norm, uni]
            prargs = [None, None, None, ecc_prargs, None]

        if b.parameterisation == 1:
            sqrta, sqrte = amp_limiter, 0.707  #(sqrt 0.5 ~ 0.707)
            sqrta, sqrte = sqrta ** 0.5, sqrte ** 2
            a_lims, e_lims = [-sqrta, sqrta], [-sqrte, sqrte]

            lims = [np.log([0.1, per_limiter*3]), a_lims, a_lims, e_lims, e_lims]
            priors = [uni, uni, uni, uni, uni]
            prargs = [None, None, None, None, None]

            b.additional_parameters = []
            pnames = ['Period', 'Amplitude', 'Phase', 'Eccentricity', 'Longitude']
            ndim = len(pnames)

            pdisplay_names = [f'{x} {b.number_}' for x in pnames]
            ppriors = [uni, uni, uni, uni, uni]
            plims = [[1e-6, per_limiter*3], [0.1, amp_limiter], [0, 2*np.pi], ecc_limits, [0, 2*np.pi]]
            pprargs = [[], [], [], ecc_prargs, []]
            phas_prior = [False, True, False, True, False]
            phas_posterior = [True, True, True, True, True]
            punits = ['(Days)', r'($\frac{m}{s}$)', '(rad)', '', '(rad)']


            for i in range(ndim):
                d0 = {'name': pnames[i],
                      'display_name': pdisplay_names[i],
                      'prior': ppriors[i],
                      'limits':plims[i],
                      'prargs':pprargs[i],
                      'has_prior':phas_prior[i],
                      'has_posterior':phas_posterior[i],
                      'unit':punits[i],
                      'fixed':None,
                      }

                b.add_additional_parameters(d0)


        if b.parameterisation == 2:
            t0_limiter = [my_data['BJD'].min(), my_data['BJD'].min() + per_limiter]

            lims = [[0.1, per_limiter], [0, amp_limiter], t0_limiter, ecc_limits, angle_limits]
            priors = [uni, uni, uni, norm, uni]
            prargs = [None, None, None, ecc_prargs, None]

        if b.parameterisation == 3:

            t0_limiter = [my_data['BJD'].min() - per_limiter, my_data['BJD'].max() + per_limiter]
            sqrte = 1  #(sqrt 0.5 ~ 0.707)
            sqrte **= 2
            e_lims = [-sqrte, sqrte]

            lims = [[0.1, per_limiter], [0, amp_limiter], t0_limiter, e_lims, e_lims]
            priors = [uni, uni, uni, uni, uni]
            prargs = [None, None, None, None, None]

            b.additional_parameters = []
            pnames = ['Eccentricity', 'Longitude']
            pdisplay_names = [f'{x} {b.number_}' for x in pnames]
            plims = [ecc_limits, angle_limits]
            pprargs = [ecc_prargs, []]
            phas_prior = [True, False]
            phas_posterior = [True, True]
            punits = ['', '(rad)']

            ndim = len(pnames)
            ppriors = [uni, uni]
            for i in range(ndim):
                d0 = {'name': pnames[i],
                      'display_name': pdisplay_names[i],
                      'prior': ppriors[i],
                      'limits':plims[i],
                      'prargs':pprargs[i],
                      'has_prior':phas_prior[i],
                      'has_posterior':phas_posterior[i],
                      'unit':punits[i],
                      'fixed':None,
                      }

                b.add_additional_parameters(d0)

            '''
            b.add_additional_priors([['Eccentricity', norm, ecc_limits, ecc_prargs],
                                   ['Longitude', uni, [0, 2*np.pi], []],
                                   ])
            '''
        if d['starmass']:
            _extracted_from_SmartLimits_122(b, uni)
        if not d['dynamics_already_included'] and d['kplan'] > 1:
            d0 = {'name': 'Hill',
                  'display_name': 'Dynamical Criteria',
                  'prior': 'Hill',
                  'limits':[None, None],
                  'prargs':[d['kplan'], d['starmass']],
                  'has_prior':True,
                  'has_posterior':False,
                  'unit':None,
                  'fixed':None,
                  }
            d['dynamics_already_included'] = True
            b.add_additional_parameters(d0)
        '''
            if d['dynamics']:
                if d['kplan'] > 1 and b.number_ == d['kplan']:
                    prarg = [d['kplan'], d['starmass']]
                    b.add_additional_priors([['Hill', 'Hill', [None, None], prarg]])
                    b.dynamics_bool = True
                else:
                    b.dynamics_bool = False
            '''

    elif b.type_ == 'Instrumental':
        jit_limits, jit_prargs = args

        mask = my_data['Flag']==b.number_
        jit_limiter = my_data[mask]['RV'].abs().max()

        lims = [[-jit_limiter, jit_limiter], [1e-5, jit_limiter]]
        priors = [uni, norm]
        prargs = [None, jit_prargs]

        if b.moav > 0:
            for _ in range(b.moav):
                lims.extend(([0.5, 1], [15, 25]))
                priors.extend((uni, uni))
                prargs.extend((None, None))
        '''
            elif b.type_ == 'AdditionalPriors':
                my_params = []

                pnames = []
                punits = []
                pvalues = []
                ppriors = []
                plimits = []

                ptypes = []
                prargs = []
                ptformargs = []
                pfixed = []
                psigma = []
                if False:
                    # All CVs go here!!!
                    sig_limiter = my_data['RV'].std(ddof=0)
                    per_limiter = my_data['BJD'].max() - my_data['BJD'].min()
                    amp_limiter = sig_limiter * np.sqrt(3)
                    angle_limits = [0, 2*np.pi]

                    ecc_limits, ecc_prargs = d['prargs']
                if True:
                    #Hill goes here
                    if d['dynamics']:
                        if d['kplan'] > 1:
                            prarg = [d['kplan'], d['starmass']]

                            pnames.append('Hill')
                            ppriors.append('Hill')
                            pvalues.append(None)
                            plimits.append([None, None])
                            punits.append(None)
                            prargs.append(prarg)
                            ptypes.append(None)
                            ptformargs.append(None)
                            pfixed.append(None)
                            psigma.append(None)

                bdim = len(pnames)
                for i in range(bdim):
                    d0 = {'name':pnames[i], 'prior':ppriors[i], 'value':pvalues[i],
                              'limits':plimits[i], 'unit':punits[i], 'prargs':prargs[i],
                              'type':ptypes[i], 'ptformargs':ptformargs[i], 'fixed':pfixed[i],
                              'sigma':psigma[i]}
                    my_params.append(Parameter(d0))
                b.list_ = np.array(my_params)
                return
            '''
    elif b.type_ != 'Acceleration':
        print(f'type_ {b.type_} not recognised. \nSmartLimits failed')

    b.set_attr('limits', lims, silent=True)
    b.set_attr('prior', priors, silent=True)
    b.set_attr('prargs', prargs, silent=True)


# TODO Rename this here and in `SmartLimits`
def _extracted_from_SmartLimits_122(b, uni):
    pnames = ['Semi-Major Axis', 'Minimum Mass']
    pdisplay_names = [f'{x} {b.number_}' for x in pnames]
    ppriors = [uni, uni]
    plims = [[1e-5, 1000], [1e-5, 1000]]
    pprargs = [[], []]
    phas_prior = [False, False]
    phas_posterior = [True, True]
    punits = ['(AU)', 'Mj']
    ndim = len(pnames)
    for i in range(ndim):
        d0 = {'name': pnames[i],
              'display_name': pdisplay_names[i],
              'prior': ppriors[i],
              'limits':plims[i],
              'prargs':pprargs[i],
              'has_prior':phas_prior[i],
              'has_posterior':phas_posterior[i],
              'unit':punits[i],
              'fixed':None,
              }

        b.add_additional_parameters(d0)

# https://en.wikipedia.org/wiki/Mathematical_operators_and_symbols_in_Unicode
# sun symbol ⊙
# earth symbol ⊕
#


# maths ≤ ≥ ∼ ∄	∊ ∏ ∑ ∫ ≪	≫
# letters 𝒜 𝒞	𝒟
# 𝒢			𝒥	𝒦			𝒩	𝒪	𝒫	𝒬		𝒮	𝒯
# 𝒰	𝒱	𝒲	𝒳	𝒴	𝒵	𝒶	𝒷	𝒸	𝒹		𝒻		𝒽	𝒾	𝒿
#	𝓀	𝓁	𝓂	𝓃		𝓅	𝓆	𝓇	𝓈	𝓉	𝓊	𝓋	𝓌	𝓍	𝓎	𝓏
#	𝓐	𝓑	𝓒	𝓓	𝓔	𝓕	𝓖	𝓗	𝓘	𝓙	𝓚	𝓛	𝓜	𝓝	𝓞	𝓟
#	𝓠	𝓡	𝓢	𝓣	𝓤	𝓥	𝓦	𝓧	𝓨	𝓩	𝓪	𝓫	𝓬	𝓭	𝓮	𝓯
#	𝓰	𝓱	𝓲	𝓳	𝓴	𝓵	𝓶	𝓷	𝓸	𝓹	𝓺	𝓻	𝓼	𝓽	𝓾	𝓿
#	𝔀	𝔁	𝔂	𝔃	𝔄	𝔅		𝔇	𝔈	𝔉	𝔊			𝔍	𝔎	𝔏
#	𝔐	𝔑	𝔒	𝔓	𝔔		𝔖	𝔗	𝔘	𝔙	𝔚	𝔛	𝔜

#   𝛢	𝛣	𝛤	𝛥	𝛦	𝛧	𝛨	𝛩	𝛪	𝛫	𝛬	𝛭	𝛮	𝛯
#	𝛰	𝛱	𝛲	𝛳	𝛴	𝛵	𝛶	𝛷	𝛸	𝛹	𝛺	𝛻	𝛼	𝛽	𝛾	𝛿
#	𝜀	𝜁	𝜂	𝜃	𝜄	𝜅	𝜆	𝜇	𝜈	𝜉	𝜊	𝜋	𝜌	𝜍	𝜎	𝜏
#	𝜐	𝜑	𝜒	𝜓	𝜔	𝜕	𝜖	𝜗	𝜘	𝜙	𝜚	𝜛	𝜜	𝜝	𝜞	𝜟
#	𝜠	𝜡	𝜢	𝜣	𝜤	𝜥	𝜦	𝜧	𝜨	𝜩	𝜪	𝜫	𝜬	𝜭	𝜮	𝜯
#	𝜰	𝜱	𝜲	𝜳	𝜴	𝜵	𝜶	𝜷	𝜸	𝜹	𝜺	𝜻	𝜼	𝜽	𝜾	𝜿
#	𝝀	𝝁	𝝂	𝝃	𝝄	𝝅	𝝆	𝝇	𝝈	𝝉	𝝊	𝝋	𝝌	𝝍	𝝎	𝝏
#	𝝐	𝝑	𝝒	𝝓	𝝔	𝝕
# ⏰	⏱	⏲	⏳ ⭐	⭑	⭒	⭓	⭔	⭕

# subscript_nums = ['₀', '₁', '₂', '₃', '₄', '₅', '₆', '₇', '₈', '₉']
# supscript_nums = ['⁰', '¹', '²', '³', '⁴', '⁵', '⁶', '⁷', '⁸', '⁹']
# ⁱ ⁺ ⁻ ⁿ ⁽	⁾
# ᵢ ₘ ₊ ₋ ₍	₎   ₍ᵢ₋₁₎
# ~𝓤()
# ~𝓝()
# ~𝓖()
# ~ 𝓙()

# ✅ ✔ ✓
# ◯
# ❎ ❌ ✘ ✗ ☒



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import setuptools

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setuptools.setup(
    name="astroemperor",
    version="0.7.2",
    author="ReddTea",
    author_email="redd@tea.com",
    description="PTMCMC sampler for exoplanet search",
    long_description=long_description,
    long_description_content_type="text/markdown",
    #url="https://github.com/pypa/sampleproject",
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    packages=setuptools.find_packages(where="src"),
    package_dir={"": "src"},
    #data_files=[(path1, [path1+'/exo_list.csv', path1+'/ss_list.csv'])],
    include_package_data=True,
    install_requires=['numpy', 'matplotlib>=3.5.1', 'PyAstronomy', 'gatspy',
                      'ipywidgets', 'tabulate', 'reddutils', 'kepler.py',
                      'emcee', 'arviz', 'corner', 'reddcolors',
                      'reddemcee'],
    python_requires=">=3.6",
)
